use std::collections::HashMap;

use winit::event::{ElementState, MouseButton, MouseScrollDelta, WindowEvent};

use super::{process_new_element_state, virtual_input::VirtualInput};

#[derive(Debug, Default)]
pub struct MouseState {
    old_position: glam::Vec2,
    position: glam::Vec2,
    delta: glam::Vec2,
    wheel_delta: glam::Vec2,
    mouse_button_state: HashMap<MouseButton, ElementState>,
}

impl MouseState {
    pub fn process_window_event(&mut self, event: &WindowEvent, virtual_input: &mut VirtualInput) {
        match event {
            WindowEvent::CursorMoved { position, .. } => {
                self.position = glam::Vec2::new(position.x as f32, position.y as f32);
            }
            WindowEvent::MouseWheel { delta, .. } => {
                self.wheel_delta += match delta {
                    MouseScrollDelta::PixelDelta(position) => {
                        glam::Vec2::new(position.x as f32, position.y as f32)
                    }
                    MouseScrollDelta::LineDelta(x, y) => glam::Vec2::new(*x as f32, *y as f32),
                };
            }
            WindowEvent::MouseInput { button, state, .. } => {
                let key_state = self.mouse_button_state.get(button);
                let new_key_state = process_new_element_state(key_state, *state);
                if let Some(new_key_state) = new_key_state {
                    self.mouse_button_state.insert(*button, new_key_state);
                    virtual_input.update_mouse_button_binding(*button, new_key_state);
                }
            }
            _ => {}
        }
    }

    pub fn update(&mut self, virtual_input: &mut VirtualInput) {
        self.delta = self.position - self.old_position;
        self.old_position = self.position;
        virtual_input.update_mouse_move_binding(&self.delta);
        virtual_input.update_mouse_wheel_binding(&self.wheel_delta);
        self.wheel_delta = glam::Vec2::ZERO;
    }
}
